@@ -1,4 +1,4 @@
-import gtk, time
+import gtk, time, random
 from gi.repository import GUPnPAV
 
 class ZhaanUI(object):
@@ -14,7 +14,10 @@ class ZhaanUI(object):
         self.source_device = None
         self.stack = []
         self.current_id = 0
-
+        self.playlist = None
+        self.current_container = None
+        self.last_state = None
+        
     def destroy(self, widget, data=None):
 	print "Exiting"
 	gtk.main_quit()
@@ -24,6 +27,7 @@ class ZhaanUI(object):
         if isinstance(item, GUPnPAV.GUPnPDIDLLiteContainer):
             self.stack.append(item.get_parent_id())
             self.current_id = item.get_id()
+            self.current_container = item
             self.upnp.load_children(self.source_device, item.get_id())
         elif isinstance(item, GUPnPAV.GUPnPDIDLLiteItem):
             self.playlist.add(item, item.get_title())
@@ -38,7 +42,17 @@ class ZhaanUI(object):
         pass
 
     def update_renderer_status(self, device, state):
-        pass
+        if device.get_udn() == self.renderer_device.get_udn():
+            # If the player stopped and we still have more music to play,
+            # then play :)
+            
+            if self.last_state == "PLAYING" and state == "STOPPED" and \
+                   (not self.empty_playlist()):
+                print "GOING NEXT"
+                self.next()                  
+
+            self.last_state = state
+
 
     def add_container(self, container):
         self.add_source_item(container, "(+) %s" % container.get_title())
@@ -193,8 +207,125 @@ class ZhaanUI(object):
                        self.renderer_device,
                        item)
 
+    def empty_playlist(self):
+        return not bool(self.playlist)
+    
+    # ------------------------
+    # ----- Player Logic -----
+    # ------------------------
+    
+    def is_random(self):
+        return self.upnp.config.get("Playback", "Randomized") == "True"
     
     def play(self, playlist, item):
+        """
+        Normal Player logic:
+        If we're in the main view and we hit play we do the following things
+        0) Save the current playing item to the playlist history
+        1) SetCurrentURI of the current renderer to the first item in the play list
+        2) Move to the control view
+        3) Remove the first item from the playlist
+        4) SetNextURI for the new first item (formerly the second item) if it exists
+
+        Randomized Player Logic:
+        To simplify things we entirely forget the concept of history and always
+        just randomly chose a next item
+        1) SetCurrentURI of a random item in the playlist
+        2) Move to control view
+        """
+        if self.is_random():
+            self.do_play(playlist, random.choice(self.playlist.items))
+            self.change_to_controller()
+            return
+
+        # 0) Save to the history
+        if self.playlist.items:
+            self.playlist.history.append(self.playlist.items[0])
+
+        # 1) setUri and begin playing
+        self.do_stop(playlist, item)
+        self.do_play(playlist, item)
+
+        # 2) Move to control view
+        self.change_to_controller()
+
+        # 3) Remove the first item from the playlist
+        self.playlist.rm(0)
+
+        # 4) Set NextURI to first item
+        if self.playlist.items:            
+            self.upnp.set_next_uri(self.source_device,
+                                   self.renderer_device,
+                                   self.playlist.items[0])
+
+
+    def prev(self, playlist=None, item=None):
+        """
+        Similar to next() there are really two modes of operation.
+        1) Standard just call previous() if we have no built in zhaan playlist
+        2) Use the built in playing history to determine what to play and use that.
+        """
+        if self.is_random():
+            return self.next(playlist, item)
+            
+        if len(self.playlist.history) < 2:
+            # See the below explanation for why we need < 2 and not < 1
+            self.do_prev(playlist, item)
+            return
+
+        # Here we have a slight dilema.  Whatever is currently in the control view
+        # is also the first item in the history.  What we really want then is the
+        # second item in the history.  We will therefore first put the current item history[-1]
+        # and the second past item history[-2] back in the playlist, and then simply call Play()
+        # Play will take off the top item in the playlist (history[-2]) and push it onto the history,
+        # and remove it from the top of the playlist
+        current_item = self.playlist.history.pop()
+        new_item = self.playlist.history.pop()
+
+        self.playlist.prepend(current_item, current_item.get_title())
+        self.playlist.prepend(new_item, new_item.get_title())
+
+        self.play(playlist, self.playlist.items[0])
+
+    def next(self, playlist=None, item=None):
+        """
+        Next Logic:
+        We need to support two paradigms of 'next'.
+        Paradigm One:
+        Zhaan has its own playlist.  By hitting next we wish to advance
+        to the next item in the playlist.  We will use this paradigm if a playlist
+        has been built.
+
+        Paradigm One Actions:
+        1) SetCurrentURI to the next item in the playlist and play
+        2) Remove the front item of the playlist
+        3) SetNextURI to the new first item (the second item)
+
+        Paradigm Two:
+        The renderer has its own notion of Next().
+
+        Paradigm Two Actions:
+        Call next() on the renderer
+        """
+
+        # Determine which paradigm to use
+        if len(self.playlist.items) == 0:
+            # Paradigm one
+            self.do_next(playlist, item)
+            return
+
+        # Paradigm two
+        # 1) Set next item and start playing
+        # 2) Remove the next item
+        # 3) Set the next URI
+        # (all of the above are done by play())
+        self.play(playlist, self.playlist.items[0])
+
+    # ---------------------------
+    # ----- Player Controls -----
+    # ---------------------------
+
+    def do_play(self, playlist, item):
         if not self.source_device or not self.renderer_device:
             print "Missing either source or destination device"
             return
@@ -207,7 +338,7 @@ class ZhaanUI(object):
                               self.renderer_device,
                               item)
         
-    def stop(self, playlist, item):
+    def do_stop(self, playlist, item):
         if not self.renderer_device:
             print "Missing Renderer"
             return
@@ -216,7 +347,7 @@ class ZhaanUI(object):
                               self.renderer_device,
                               self.playing_item)
 
-    def pause(self, playlist, item):
+    def do_pause(self, playlist, item):
         if  not self.renderer_device:
             print "Missing Renderer"
             return
@@ -225,7 +356,7 @@ class ZhaanUI(object):
                               self.renderer_device,
                               self.playing_item)
 
-    def prev(self, playlist, item):
+    def do_prev(self, playlist, item):
         if not self.renderer_device:
             print "Missing Renderer"
             return
@@ -234,7 +365,7 @@ class ZhaanUI(object):
                               self.renderer_device,
                               self.playing_item)
 
-    def next(self, playlist, item):
+    def do_next(self, playlist, item):
         if not self.renderer_device:
             print "Missing Renderer"
             return
@@ -243,12 +374,12 @@ class ZhaanUI(object):
                               self.renderer_device,
                               self.playing_item)
 
-    def seek(self, abs_time):
+    def do_seek(self, abs_time):
         self.upnp.seek_object(self.source_device,
                               self.renderer_device,
                               self.playing_item,
                               abs_time)
 
-    def set_volume(self, volume):
+    def do_set_volume(self, volume):
         self.upnp.set_volume(self.renderer_device,
                              volume)
